package com.jdbernard.timestamper

import com.jdbernard.timestamper.core.TimelineMarker
import com.jdbernard.timestamper.core.TimelineProperties

class TimeStamperMainController {
    // these will be injected by Griffon
    def model
    def view

    void mvcGroupInit(Map args) {

        if (logger.isTraceEnabled())
            logger.trace("Initializeing TimeStamperMain MVC...")

        def thisMVC = ['model': model, 'view': view, 'controller': this]

        model.notesDialogMVC = buildMVCGroup('NotesDialog', 'notesDialog',
            'mainMVC': thisMVC)

        model.punchcardDialogMVC = buildMVCGroup('PunchcardDialog',
            'punchcardDialog', 'mainMVC': thisMVC)

        // load application properties
        Properties prop = new Properties()
        String userHomeDir = System.getProperty('user.home')
        model.configFile = new File(userHomeDir, ".timestamperrc")
        if (!model.configFile.exists()) model.configFile.createNewFile()

        try { model.configFile.withInputStream { prop.load(it) } }
        catch (IOException ioe) { 
            logger.error('Unable to load configuration', ioe)
        }

        model.config = prop

        // load the last used timeline file
        String lastUsed = model.config.getProperty('lastUsed', null)
        if (lastUsed == null) {
            lastUsed = 'timeline.default.properties'
            model.config.setProperty('lastUsed', lastUsed)
        }
        File propertyFile = new File(lastUsed)
        if (!propertyFile.exists()) propertyFile.createNewFile()

        model.timelineProperties = new TimelineProperties(propertyFile)

        // load the main timeline
        model.timeline = model.timelineProperties.timeline

        // load the last marker
        model.currentMarker = model.timeline.getLastMarker(new Date())
        
        if (logger.isTraceEnabled())
            logger.trace("TimeStamperMain MVC initialization complete.")

    }

    def exitGracefully = { evt = null ->
    
        if (logger.isTraceEnabled()) logger.trace("Exiting gracefully.")

        // save config
        try { model.configFile.withOutputStream { out ->
                model.config.store(out, null) } }
        catch (IOException ioe) {
            logger.error("Unable to save the configuration file", ioe)
        }

        // save timeline and properties
        model.timelineProperties.save()

        if (logger.isTraceEnabled()) logger.trace("Completed graceful shutdown.")

        app.shutdown()
    }

    def newTask = { mark ->
        model.currentMarker = new TimelineMarker(new Date(), mark,
            "No comments.")
        model.timeline.addMarker(model.currentMarker)
    }

}
