package com.jdbernard.timestamper

import java.awt.Color
import java.awt.Font
import java.awt.Point
import java.awt.Rectangle
import java.awt.Toolkit
import java.awt.event.KeyEvent
import javax.swing.BoxLayout
import javax.swing.Timer
import com.jdbernard.timestamper.core.Timeline
import net.miginfocom.swing.MigLayout

/* ========== *
 * GUI Events *
 * ========== */

taskTextFieldChanged = { evt = null ->
    if (evt.keyCode == KeyEvent.VK_ENTER) {
        taskTextField.font = taskBoldFont
        controller.newTask(taskTextField.text)
    }

    else if (evt.keyCode == KeyEvent.VK_ESCAPE) {
        taskTextField.font = taskBoldFont
        taskTextField.text = model.currentMarker.mark
    }

    else if (!evt.isActionKey())
        taskTextField.font = taskThinFont
}

showToolsMenu = { evt = null ->

}

showNotes = { evt = null ->
    model.notesDialogMVC.view.dialog.visible = notesVisibleButton.selected
}

showPunchcard = { evt = null ->

}

mousePressed = { evt = null ->
    mousePressRelativeToFrame = evt?.point
}

mouseDragged = { evt = null ->
    GUIUtil.componentDragged(frame, evt, mousePressRelativeToFrame,
        new Rectangle(Toolkit.defaultToolkit.screenSize))
}

/* ============== *
 * GUI Definition *
 * ============== */

updateTimer = new Timer(1000, action(name: 'GUI Refresh', closure: {
    Date currentTime = new Date()
    currentTimeLabel.text = Timeline.shortFormat.format(currentTime)
    if (model.currentMarker != null) {
        long seconds = currentTime.time - model.currentMarker.timestamp.time
        seconds /= 1000
        long minutes = seconds / 60
        seconds = seconds % 60
        long hours = minutes / 60
        minutes %= 60
        long days = hours / 24
        hours %= 24

        StringBuilder sb = new StringBuilder()
        if (days > 0) sb.append(days + "day ")
        if (hours > 0) sb.append(hours + "hr ")
        if (minutes > 0) sb.append(minutes + "min ")
        sb.append(seconds + "sec")

        totalTimeLabel.text = sb.toString()
    } else totalTimeLabel.text = ""
}))

updateTimer.start()

frame = application(title:'TimeStamper',
  //size:[320,480],
  pack:true,
  //location:[50,50],
  undecorated:true,
  locationByPlatform:true,
  iconImage: imageIcon('/appointment-new-32x32.png').image,
  iconImages: [imageIcon('/appointment-new-32x32.png').image,
               imageIcon('/appointment-new-16x16.png').image],
  componentMoved: { evt -> model.absoluteLocation = frame.location }
) {
    panel(
      border:lineBorder(color:Color.BLACK, thickness:1, parent:true),
      layout: new MigLayout('insets 0 5 0 0, fill','', '[]0[]0[]'),
      mousePressed: mousePressed,
      mouseDragged: mouseDragged
    ) {
        def mainFont = new Font(Font.SANS_SERIF, Font.BOLD, 12)
        def timeFont = new Font(Font.SANS_SERIF, Font.BOLD, 14)
        label("Current task started at ", font: mainFont)
        label(constraints: 'align leading', font: timeFont,
            foreground: [0, 102, 102],
            text: bind(source: model, sourceProperty: 'currentMarker',
                sourceValue: {
                    model.currentMarker == null ? "00:00:00" :
                    Timeline.shortFormat.format(model.currentMarker.timestamp)
                }))

        panel(constraints: 'alignx trailing, aligny top, wrap') {
            boxLayout(axis: BoxLayout.X_AXIS)
            button(actionPerformed: showToolsMenu,
                icon: imageIcon('/16-tool-a.png'),
                rolloverIcon: imageIcon('/16-tool-a-hover.png'),
                border: emptyBorder(0),
                contentAreaFilled: false,
                hideActionText: true)
            button(actionPerformed: controller.&exitGracefully,
                icon: imageIcon('/16-em-cross.png'),
                rolloverIcon: imageIcon('/16-em-cross-hover.png'),
                border: emptyBorder(0),
                contentAreaFilled: false,
                hideActionText: true)
        }

        taskTextField = textField("Task name",
            constraints: "growx, span 2, w 250::",
            keyReleased: taskTextFieldChanged,
            text: bind(source: model, sourceProperty: 'currentMarker',
                sourceValue: { model.currentMarker.mark }))

        taskThinFont = taskTextField.font
        taskBoldFont = taskTextField.font.deriveFont(Font.BOLD)

        panel(constraints: 'alignx leading, aligny top, gapright 5px, wrap') {
            boxLayout(axis: BoxLayout.X_AXIS)
            notesVisibleButton = toggleButton(
                actionPerformed: showNotes,
                icon: imageIcon('/16-em-pencil.png'),
                hideActionText: true,
                border: emptyBorder(4))
            punchcardVisibleButton = toggleButton(
                actionPerformed: showPunchcard,
                icon: imageIcon('/16-file-archive.png'),
                hideActionText: true,
                border: emptyBorder(4))
        }

        totalTimeLabel = label("", constraints: 'alignx leading',
            font: timeFont, foreground: [0, 153, 0])

        currentTimeLabel = label("00:00:00", constraints: 'align trailing',
            font: timeFont, foreground: [204, 0, 0])
    }
}


